var util = function() {
    return {
		// ___ Edge必須処理 START ____________________________________________________________
        // 本番環境では、サーバ側で処理する
        nextAction: function(){
			self.location = params['next_path'];
        },
        // SEを再生
        playSE: function(filename) {
            if (IS_DEV) {
                return;
            }
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "playSE:" + filename;
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.playSE(filename);
            }
        },
        
        // BGMを再生
        playBGM: function(filename) {
            if (IS_DEV) {
                return;
            }
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "playBGM:" + filename;
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.playBGM(filename);
            }
        },        
        
        // BGMを停止（ページ遷移直前に実行しても停止できないので注意）
        stopBGM: function() {
            if (IS_DEV) {
                return;
            }            
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "stopBGM:";
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.stopBGM();    
            }
        },
		
		// ___ Edge必須処理 END ________________________________________________________________
        
		// 本番環境では、サーバ側で処理する
        effectStart: function(){
            util.log("util.effectStart()");            
        },
		
        // アニメーション初期化時に予め行なっておきたいこと
        init: function() {
            // 端末名
            util.getDeviceName();
        },
        
        // アニメーションを中断
        pause: function(element) {            
            if (typeof element.id !== "undefined" && element.id != "") {    
                element.style.webkitAnimationPlayState = 'paused';
            }
        },    
        
        // アニメーションを再生
        run: function(element) {            
            if (typeof element.id !== "undefined" && element.id != "") {                
                element.style.webkitAnimationPlayState = 'running';
            }
        },        
        
        // アニメーションを停止
        interruptAnimation: function(element, callback) {
            element.style.webkitAnimationName = "none";        
            if (callback) {
                element.removeEventListener("webkitAnimationEnd", callback, false);
            } 
        },
        
        // Senchaが吐き出したアニメーションを再生
        playAsExported: function(element) {            
            element.style.webkitAnimationName = "ani-" + element.id;                    
        },
        
        // アニメーションをオーバーライドかつ停止
        noneAnimation: function(element) {
            element.style.webkitAnimationName = "none";    
        },
        
        // 無限再生アニメーションを実行
        infiniteAnimation: function(element, animationName, interval, callback) {
            element.style.webkitAnimationName = "none";    
            setTimeout(function() { 
                util.setCallbackToAnimationEnd(element, function() {                        
                    if (callback != null) {
                        callback();    
                    }                                        
                    util.infiniteAnimation(element, animationName, interval, callback);    
                });            
                element.style.webkitAnimationDelay = interval.toString() + "s";
                element.style.webkitAnimationName = animationName;
            }, 1);                            
        },
        
        // アニメーション完了状態までスキップ
        skipAnimationToEnd: function(element, animation) {        
            util.noneAnimation(element);
            element.style.webkitAnimationDuration = "0.0s";        
            setTimeout(function() {                
                animation == undefined ? 
                    util.playAsExported(element) : 
                    element.style.webkitAnimationName = animation;                
                element.style.webkitAnimationPlayState = 'running';
            }, 1);            
        },        
        
        // アニメーションを指定して再生（pauseしていたのを再生想定）
        playThisAnimation: function(element, duration, name, timing) {
            element.style.webkitAnimationName = "none";
            setTimeout(function() {
                if (timing != undefined) {
                    element.style.webkitAnimationTimingFunction = timing;
                }                
                element.style.webkitAnimationDuration = duration + "s";
                element.style.webkitAnimationName = (name != undefined ? name : "ani-" + element.id);
                if (element.style.webkitAnimationPlayState != "running") {
                    element.style.webkitAnimationPlayState = "running";
                }            
            }, 1);        
        },
        
        // アニメーションした際に効果が残るようにする
        keepAnimated: function(element) {            
            element.style.webkitAnimationFillMode = "both";
        },
        
        // DOM要素を下に辿り、指定の関数を実行していく
        recurseDomChildren: function(start, func) {    
            var nodes;
            if (start.childNodes) {
                nodes = start.childNodes;
                util.loopNodeChildren(nodes, func);
            }
        },
        
        // ノードの子どもに対して再帰処理で指定関数を実行
        loopNodeChildren: function(nodes, func) {
            var node;
            for (var i = 0; i < nodes.length; i++) {
                node = nodes[i];
                if (func) {
                    func(node);
                }
                if (node.childNodes) {
                    util.recurseDomChildren(node, func);
                }
            }
        },
        
        // ノード要素のトレースを行う
        outputNode: function(node) {
            var whitespace = /^\s+$/g;
            if (node.nodeType === 1) {
                util.log("element: " + node.tagName);
                
            } else if (node.nodeType === 3) {
                //clear whitespace text nodes
                node.data = node.data.replace(whitespace, "");
                if (node.data) {
                    util.log("text: " + node.data); 
                }  
            }  
        },
        
        // アニメーション前のコールバックをセット
        setCallbackToAnimationStart: function(element, callback) {
            element.addEventListener("webkitAnimationStart", function() {            
                element.removeEventListener("webkitAnimationStart", arguments.callee, false);
                callback();                
            }, false);
        },
        
        // アニメーション後のコールバックをセット
        setCallbackToAnimationEnd: function(element, callback) {
            element.addEventListener("webkitAnimationEnd", function() {
                element.removeEventListener("webkitAnimationEnd", arguments.callee, false);
                callback();
            }, false);
        },    
        
        // アニメーション後に対して登録したコールバックを外す
        removeCallbackFromAnimationEnd: function(element, callback) {
            element.removeEventListener("webkitAnimationEnd", callback, false);
        },    
        
        // タッチ始めに対してコールバックをセット
        setCallbackToTouchStart: function(element, callback, removeListener) {                                
            var eventName = (!IS_DEV && (userAgent == ACCESS_FROM_IOS || userAgent == ACCESS_FROM_ANDROID)) ? 
                "touchstart" : "mousedown";
            element.addEventListener(eventName, function() {    
                if (removeListener) {
                    element.removeEventListener(eventName, arguments.callee, false);
                }                                
                callback();                
            }, false);
        },
        
        // タッチ始めに対して登録したコールバックを外す
        removeCallbackFromTouchStart: function(element, callback) {                    
            var eventName = (!IS_DEV && (userAgent == ACCESS_FROM_IOS || userAgent == ACCESS_FROM_ANDROID)) ? 
                "touchstart" : "mousedown";
            element.removeEventListener(eventName, callback, false);    
        },
        
        // タッチ後に対してコールバックをセット
        setCallbackToTouchEnd: function(element, callback, removeListener) {                
            var eventName = (!IS_DEV && (userAgent == ACCESS_FROM_IOS || userAgent == ACCESS_FROM_ANDROID)) ? 
                "touchend" : "mouseup";
            element.addEventListener(eventName, function() {                
                if (removeListener) {
                    element.removeEventListener(eventName, arguments.callee, false);
                }                                
                callback();                
            }, false);
        },
        
        // この関数実行前にセットした値を即実行
        setImmediateAction: function(element) {
            // setting 0ms won't work.
            element.style.webkitAnimationDuration = "1ms";
            element.style.webkitAnimationDelay = "1ms";        
        },
        
        // ScaleXの正確な値を取得    
        getPreciseScaleX: function(element) {
            var computedStyle = window.getComputedStyle(element, null).getPropertyValue("-webkit-transform");
            var cF = computedStyle.indexOf("(");
            var cL = computedStyle.indexOf(",");            
            return computedStyle.substring(cF+1, cL);
        },
        
        // Tramsformの正確な値を取得
        getPreciseTransformStyle: function(element) {
            // always returns same value
            var transformStyle = element.style.webkitTransform;
            var tF = transformStyle.lastIndexOf("(");
            var tL = transformStyle.lastIndexOf(")");            
            var tStyle = transformStyle.substring(tF+1, tL);
            var tStyleArr = tStyle.split(",");
            var tV = tStyleArr[0];        
            return transformStyle.substring(0, tF+1) + util.getPreciseScaleX(element) + ", " + tStyleArr[1] + ", " + tStyleArr[2] + ")";
        },
        
        // Tramsformの指定プロパティ値を取得
        getTransformStyle: function(element, property) {            
            var style = window.getComputedStyle(element, '').getPropertyValue("-webkit-transform");            
            var values = style.substring(style.lastIndexOf("(")+1, style.lastIndexOf(")")).split(",");        
            var is3D = values.length == 16;
            var result;
            switch (property.toLowerCase()) {
                case "x":                
                    result = is3D ? values[12] : values[4];
                    break;
                case "y":
                    result = is3D ? values[13] : values[5];
                    break;
                case "z":
                    result = is3D ? values[14] : 0;
                    break;    
                case "scalex":
                    result = values[0];
                    break;    
                case "scaley":
                    result = is3D ? values[5] : values[3];
                    break;    
                case "scalez":
                    result = is3D ? values[10] : 1;
                    break;
                default:
                    result = 0;
                     break;
            }        
            return Number(result);
        },
        
        // keyframes宣言をCSSに追加
        setKeyFramesRule: function(element, props, ruleName) {                    
            var alpha = props["alpha"] != undefined ? props["alpha"] : element.style.opacity;            
            var rule = document.createTextNode('@-webkit-keyframes '+ruleName+' {'+
            '0%   { -webkit-transform: '+ util.getTransformStr(element) +' opacity: '+element.style.opacity+'}'+
            '100% { -webkit-transform: '+ util.getTransformStr(element, props) +' opacity: '+alpha+'}'+
            '}');        
            util.addCssRule(rule);        
        },
        
        // CSS宣言をHTMLに追記
        setCssDeclaration: function(ruleName, props, width, height, fillMode) {            
            var propsAvailable = props != undefined;                        
            var x = propsAvailable && props["x"] != undefined ? props["x"] : 0;
            var y = propsAvailable && props["y"] != undefined ? props["y"] : 0;
            var z = propsAvailable && props["z"] != undefined ? props["z"] : 0;
            var scaleX = propsAvailable && props["scaleX"] != undefined ? props["scaleX"] : 1;
            var scaleY = propsAvailable && props["scaleY"] != undefined ? props["scaleY"] : 1;
            var scaleZ = propsAvailable && props["scaleZ"] != undefined ? props["scaleZ"] : 1;
            var alpha = propsAvailable && props["alpha"] != undefined ? props["alpha"] : 1;
                        
            var rule = document.createTextNode('#'+ruleName+' {'+
                '-webkit-transform: translate3d('+x+'px, '+y+'px, '+z+'px) scale3d('+scaleX+', '+scaleY+', '+scaleZ+');' + 
                'width: '+width+'px;' + 
                'height: '+height+'px;' + 
                'top: 0;' + 
                'left: 0;' +     
                'opacity: '+alpha+';' +                 
                '-webkit-animation-fill-mode: '+fillMode+';' +                 
            '}');                    
            util.addCssRule(rule);            
        },
        
        // transformの宣言値を取得
        getTransformStr: function(element, props) {        
            var getValue = function(target) {            
                return (props != undefined && props[target] != undefined) ? 
                    props[target] : 
                    util.getTransformStyle(element, target);
            }                    
            var str = "translate3d("+
                getValue("x")+"px, "+
                getValue("y")+"px, "+
                getValue("z")+"px) scale3d("+
                getValue("scaleX")+", "+
                getValue("scaleY")+", "+
                getValue("scaleZ")+");";
            return str;
        },
        
        // 指定のtramsform値までアニメーション
        animateTransformValues: function(element, props, duration) {                        
            // tmp rule
            var ruleName = "tmp" + util.getRandomNumber();
            util.setKeyFramesRule(element, props, ruleName);            
            // animate
            util.playThisAnimation(element, duration, ruleName);
        },
        
        // ランダムな数字を返す
        getRandomNumber: function() {
            return Math.ceil(new Date().getTime() * Math.random());
        },
        
        // 指定のtramsform値を即時反映
        setTransformValues: function(element, props) {                        
            // check size
            var style = window.getComputedStyle(element, null);            
            // make and apply rule
            util.setCssDeclaration(
                element.id, 
                props != undefined ? props : null, 
                (props != undefined && props["width"] != undefined) ? props["width"] : style.getPropertyValue("width"), 
                (props != undefined && props["height"] != undefined) ? props["height"] : style.getPropertyValue("height"),
                "both");
        },
        
        // サウンド再生の遅延実行
        playThisSound: function(se, wait, callbackExecuter, element) {                                    
            var playIfOkay = function() {                    
                if (callbackExecuter != undefined) {
                    if (callbackExecuter.executable) {
                        util.playSE(se);
                    } else {
//                        util.log("wont't play this: " + se);                            
                    }    
                } else {
                    util.playSE(se);                            
                }
            }            
            if (element != undefined) {
                util.setCallbackToAnimationEnd(element, function() {
                    setTimeout(function() {
                        playIfOkay();
                    }, wait);
                });            
            } else {
                setTimeout(function() {
                    playIfOkay();
                }, wait);
            }
        },
        
        // アニメーション後のコールバックをセット
        setCallbackToAnimationEnd2: function(element, callback) {
            element.addEventListener("webkitAnimationEnd", function() {
                element.removeEventListener("webkitAnimationEnd", arguments.callee, false);
                callback();
            }, false);
        },    
        
        // 設定文字数以上の文字列を第三引数の値に置換
        replaceExtraStr: function(name, max, replacement) {            
            if (name.length > max) {
                name = name.substr(0, max);
                name += replacement;
            }        
            return name;
        },
        
        // CSS宣言を追加
        addCssRule: function(rule) {
            var style = document.createElement('style');
            style.type = 'text/css';
            style.appendChild(rule);
            document.getElementsByTagName("head")[0].appendChild(style);
        },
        
        // 外部CSSをロード
        loadCSS: function(path) {
            
            // get matched group string
//            var matched = path.match(/\/(.+)\.css/)[1].toString();            
//            util.log("matched: " + matched);
            
            var link  = document.createElement('link');
            link.id   = path;
            link.rel  = 'stylesheet';
            link.type = 'text/css';
            link.href = path;
            link.media = 'all';
            document.getElementsByTagName('head')[0].appendChild(link);
        },
        
        // メッセージダイアログを表示
        showMessage: function(text) {
            util.hideMessage();
            setTimeout(function() {                        
                var messageBox = $("messageBox");
                var messageArrow = $("messageArrow");
                messageBox.style.display = "block";
                messageArrow.style.display = "block";        
                util.setCallbackToAnimationEnd(messageBox, function() {
                    $("messageTF").innerText = text;
                    util.playAsExported(messageArrow);
                });
                util.playAsExported(messageBox);            
            }, 1);    
        },
        
        // メッセージダイアログを非表示
        hideMessage: function() {
            $("messageTF").innerText = "";
            $("messageBox").style.display = "none";
            $("messageArrow").style.display = "none";
        },
        
        // タップで次ページに遷移
        tapToNextPage: function(next_path) {                
            var func = typeof(next_path) == "string" ? function(){self.location = next_path;} : next_path;
            util.setCallbackToTouchStart(document, func , true);
        },        

        // 画像を先にロードしておく
        prepareImg: function(tag, src, parent) {
            var el = document.createElement("img"); 
            el.id = tag;
            el.src = src;
            el.style.opacity = 0;
            parent.appendChild(el);
            return el;
        },
        
        // 属性アイコンのファイル名取得
        getAttributeImg: function(id) {    
            var img;        
            switch (Number(id)) {
                case 1:
                    img = "SkillAttributeAffects_Justice.png";                
                    break;
                case 2:
                    img = "SkillAttributeAffects_Chaos.png";
                    break;
                case 3:
                    img = "SkillAttributeAffects_Genesis.png";
                    break;
            }                
            return img;
        },
        
        // 効果アイコンのイメージ名を一部取得
        getAffectedImgName: function(affectedType) {
            var str;
            switch (Number(affectedType)) {
                case 0:
                str = "Up";
                break;
                case 1:
                str = "Down";
                break;
                case 2:
                str = "Both";
                break;            
            }
            return str;
        },
        
        // メニューを表示
        showMenu: function() {
            if (IS_DEV) {
                return;
            }            
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "showMenu:";
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.showMenu();    
            }
        },
        
        // メニューを非表示
        hideMenu: function() {
            if (IS_DEV) {
                return;
            }            
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "hideMenu:";
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.hideMenu();    
            }
        },
        
        // 接続中の端末名
        deviceName: "",
        
        // 端末名を取得
        getDeviceName: function() {
            if (IS_DEV) {
                return;
            }
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "getDeviceName:";
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.getDeviceName();    
            }    
        },
        
        // 端末名をセット
        setDeviceName: function(str) {
            util.deviceName = str;    
        },
        
        // 低スペック端末か
        isLowSpecDevice: function() {                
            return (navigator.userAgent.indexOf('iPod') > 0 ||
                util.deviceName == "iPhone3,1"// iPhone4
                );
        },
        
        // OS6以上かチェック
        isOS6: function() {        
            var flg = false;        
            if(/(iPhone|iPod|iPad)/i.test(navigator.userAgent)) {             
                if(/OS [2-5]_\d(_\d)? like Mac OS X/i.test(navigator.userAgent)) {  
                    // iOS 2-5 
                    flg = false;
                } else {
                    // iOS 6 or Newer 
                    flg = true;
                }
            }        
            return flg;
        },
        
        // ログ用テキストをネイティブ側で表示        
        showToast: function(txt) {
            util.log("showToast() " + txt);
            if (IS_DEV) {
                return;
            }
            if (userAgent == ACCESS_FROM_IOS) {
                document.location = ORDER_TO_OBJ_C + "showToast:" + txt;
            } else if (userAgent == ACCESS_FROM_ANDROID) {
                window.Android.showToast(txt);
            }
        },

        // コンソールにログ表示
        log: function(arg) {
            if (SHOW_LOG) {    
                if ($("logTxt") != undefined) {
                    var el = $("logTxt");
                    if (el.style.display != "block") {
                        el.style.display = "block";
                    }                
                    el.innerText += arg + "\n";    
                    // scroll to bottom
                    setTimeout(function() {
                        el.scrollTop = el.scrollHeight;    
                    }, 100);                    
                }
            } else {
                if (IS_DEV) {
                    console.log(arg);        
                }    
            }
        }
        
    }
}();

	
/* 演出用のUtilは以下 */
var effect_util = function() {
    return {
		// EFFECT_DEV_MODE 削除不可（挙動管理のためにとても重要）
		EFFECT_DEV_MODE:"deliver", // deliver : deliverフォルダでの動作である , server: serverフォルダでの動作である
		
		// ________________________________ ここまで削除しない ______________________
		
		// logMessage
        logMessage: function( str ){
            console.log( "effect_util.logMessage() : " + str );
        }
    }
}();